# coding: utf-8

import logging
import urlparse

import requests

from tionix_scheduler_client.settings import CONF


logger = logging.getLogger(__name__)

if CONF.HOST.lower().startswith('http://'):
    SCHEDULER_URL = '{0}:{1}'.format(CONF.HOST, CONF.PORT)
else:
    SCHEDULER_URL = '{0}://{1}:{2}'.format(CONF.PROTOCOL, CONF.HOST,
                                           CONF.PORT)


def join_url(url, prefix=None):
    """Join url with prefix."""
    if prefix and not prefix.endswith('/'):
        prefix += '/'

    return urlparse.urljoin(url, prefix or '')


class SchedulerClientHttpException(Exception):
    """
    Raise this Exception when scheduler rest api response with errors
    """
    def __init__(self, msg, status_code):
        super(SchedulerClientHttpException, self).__init__(msg)

        self.msg = msg
        self.status_code = status_code

    def __str__(self):
        return self.msg


class Client(object):
    """
    Class-manager of scheduler's rest api.
    """
    BASENAME = 'timetable/'
    BASE_URL = urlparse.urljoin(SCHEDULER_URL, BASENAME)
    MODULE_BASENAME = 'tnx-scheduler/'
    MODULE_BASE_URL = urlparse.urljoin(SCHEDULER_URL, MODULE_BASENAME)

    def __init__(self, headers=None):
        self.headers = headers or {}
        self.headers['content-type'] = 'application/json'

    def add_header(self, key, value):
        self.headers[key] = value

    def _get(self, prefix=None, params=None):
        """
        Perform a host "action".
        """
        url = join_url(self.BASE_URL, prefix)

        logger.debug(u'Sending GET {0} data:{1}'.format(url, params))
        r = requests.get(url, params=params or {}, headers=self.headers)

        # pylint: disable=no-member
        if r.status_code not in (requests.codes.OK, requests.codes.ACCEPTED):
            raise SchedulerClientHttpException(r.content, r.status_code)

        logger.info(u'GET {0} successed with {1}:{2}'.format(
            url, r.status_code, r.content.decode('utf-8')))

        return r.json()

    def _delete(self, prefix=None, params=None):
        """
        Perform delete request.
        """
        url = join_url(self.BASE_URL, prefix)

        logger.debug(u'Sending DELETE {0} data:{1}'.format(url, params))
        r = requests.delete(url, json=params, headers=self.headers)

        # pylint: disable=no-member
        if r.status_code != requests.codes.NO_CONTENT:
            raise SchedulerClientHttpException(r.content, r.status_code)

        logger.info(u'DELETE {0} successed with {1}:{2}'.format(
            url, r.status_code, r.content.decode('utf-8')))

        return True

    def _post(self, prefix=None, params=None):
        """
        Perform post request.
        """
        url = join_url(self.BASE_URL, prefix)

        logger.debug(u'Sending POST {0} data:{1}'.format(url, params))
        r = requests.post(url, json=params, headers=self.headers)

        # pylint: disable=no-member
        if r.status_code not in (requests.codes.OK, requests.codes.CREATED):
            raise SchedulerClientHttpException(r.content, r.status_code)

        logger.info(u'POST {0} successed with {1}:{2}'.format(
            url, r.status_code, r.content.decode('utf-8')))

        return r.json()

    def _put(self, prefix=None, params=None):
        """
        Perform put request.
        """
        url = join_url(self.BASE_URL, prefix)

        logger.debug(u'Sending PUT {0} data:{1}'.format(url, params))
        r = requests.put(url, json=params, headers=self.headers)
        logger.info(u'PUT {0} successed with {1}:{2}'.format(
            url, r.status_code, r.content.decode('utf-8')))

        return r.json()

    def _patch(self, prefix=None, params=None):
        """
        Perform patch request.
        """
        url = join_url(self.BASE_URL, prefix)

        logger.debug(u'Sending PATCH {0} data:{1}'.format(url, params))
        r = requests.patch(url, json=params, headers=self.headers)

        logger.info(u'PATCH {0} successed with {1}:{2}'.format(
            url, r.status_code, r.content))

        return r.json()

    def get_tasks_list(self, object_type=None, groups=None, locale=None):
        """
        List of available tasks to schedule.
        """
        tasks_list = 'tasks_list/'
        params = {
            'type': object_type,
            'groups': groups,
            'locale': locale,
        }
        return self._get(tasks_list, params)

    def get_monitoring_tasks_list(self, params, use_post=False):
        """
        List of celery running tasks from pending state.
        """
        if use_post:
            method = self._post
        else:
            method = self._get
        return method('tasks_monitoring/', params)

    def delete_monitoring_tasks_list(self, params):
        """
        Delete found tasks by params
        """
        return self._delete('tasks_monitoring/', params)

    def get_object_types_list(self):
        """
        List of object types available for schedule.
        """
        return self._get('task_objects/')

    def create_timetable(self, data):
        """
        Creates timetable with periodic task.
        """
        return self._post(params=data)

    def get_timetable(self, timetable_id):
        """
        Gets one timetable by ID.
        """
        return self._get(str(timetable_id))

    def update_timetable(self, timetable_id, data):
        """
        Updates timetable with periodic task.
        """
        timetable_id = str(timetable_id)

        partial_update = ('interval' not in data) and ('crontab' not in data)
        if partial_update:
            # updates only timetable-record.
            result = self._patch(timetable_id, data)
        else:
            # update timetable-record removing previous periodic task and
            # adding new periodic task with giving params
            result = self._put(timetable_id, data)

        return result

    def delete_timetable(self, timetable_id):
        """
        Delete timetable with periodic task.
        """
        return self._delete(str(timetable_id))

    @property
    def info(self):
        """
        Get module info
        :return:
        """
        r = requests.get(self.MODULE_BASE_URL, headers=self.headers)
        return r.json().get('info', {})

    @property
    def version(self):
        """
        Get version of scheduler
        :return:
        """
        return self.info.get('version')

    def get_task_results(self, timetable_id, limit=10, offset=0):
        """
        List of task results.
        """
        params = {
            'timetable_id': timetable_id,
            'limit': limit,
            'offset': offset,
        }
        return self._get('task_results/', params)
